from django import forms
from django.db import models
import bleach

from prose.widgets import RichTextEditor


ALLOWED_TAGS = [
    "p",
    "ul",
    "ol",
    "li",
    "strong",
    "em",
    "div",
    "span",
    "a",
    "blockquote",
    "pre",
    "figure",
    "figcaption",
    "br",
    "code",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "picture",
    "source",
    "img",
    "del",
]
ALLOWED_ATTRIBUTES = [
    "alt",
    "class",
    "id",
    "src",
    "srcset",
    "href",
    "media",
]


class RichTextField(models.TextField):
    def formfield(self, **kwargs):
        kwargs["widget"] = RichTextEditor
        return super().formfield(**kwargs)

    def pre_save(self, model_instance, add):
        raw_html = getattr(model_instance, self.attname)
        if not raw_html:
            return raw_html

        sanitized_html = bleach.clean(
            raw_html, tags=ALLOWED_TAGS, attributes=ALLOWED_ATTRIBUTES
        )
        return sanitized_html


class DocumentContentField(RichTextField):
    pass



from django.db import models
from django.utils.html import strip_tags

from prose.fields import DocumentContentField


class AbstractDocument(models.Model):
    content = DocumentContentField()

    def get_plain_text_content(self):
        return strip_tags(self.content)

    def __str__(self):
        plain_text = self.get_plain_text_content()

        if len(plain_text) < 32:
            return plain_text

        return f"{plain_text[:28]}..."

    class Meta:
        abstract = True


class Document(AbstractDocument):
    pass



from django.core.files.storage import default_storage
from django.http import JsonResponse


def upload_attachment(request):
    if request.method == "POST":
        attachment = request.FILES["file"]
        key = request.POST["key"]
        path = f"prose/{key}"
        default_storage.save(path, attachment)
        payload = {
            "url": default_storage.url(path),
        }
        return JsonResponse(payload, status=201)



# Generated by Django 2.1.4 on 2019-01-05 16:09

from django.db import migrations, models
import prose.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Document",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", prose.fields.DocumentContentField()),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['prose', 'prose.migrations']

package_data = \
{'': ['*'], 'prose': ['static/prose/*', 'templates/prose/forms/widgets/*']}

install_requires = \
['bleach>=4.0', 'django>=3.2']

setup_kwargs = {
    'name': 'django-prose',
    'version': '1.2.2',
    'description': 'Wonderful rich text for Django',
    'long_description': '# Django Prose\n\n![PyPI - Downloads](https://img.shields.io/pypi/dw/django-prose?color=purple) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-prose)\n\nDjango Prose provides your Django applications with wonderful rich-text editing capabilities.\n\n## Requirements\n\n- Python 3.8 or later\n- Django 3.2 or later\n- Bleach 4.0 or later\n\n## Getting started\n\nTo get started with Django Prose, all you need to do is follow **just four steps**.\n\n1. **Install `django-prose`**\n    \n    We use and suggest using Poetry, although Pipenv and plain pip will work seamlessly as well\n    \n    ```console\n    poetry add django-prose\n    ```\n\n2. **Add to `INSTALLED_APPS`**\n    \n    Add `prose` in your Django project\'s installed apps (example: [`example/example/settings.py`](https://github.com/withlogicco/django-prose/blob/9e24cc794eae6db48818dd15a483d106d6a99da0/example/example/settings.py#L46)):\n    \n    ```python\n    INSTALLED_APPS = [\n        # Django stock apps (e.g. \'django.contrib.admin\')\n    \n        \'prose\',\n    \n        # your application\'s apps\n    ]\n    ```\n\n3. **Run migrations**\n\n    This is required so you can use Django Prose\'s built-in Document model:\n  \n    ```console\n    python manage.py migrate prose\n    ```\n\n4. **Include URLs**\n\n    You need to edit the main `urls.py` file of your Django project and include `prose.urls`:\n    \n    ```python\n    urlpatterns = [\n        path(\'admin/\', admin.site.urls),\n        # other urls ...\n        path("prose/", include("prose.urls")),\n    ]\n    ```\n\nNow, you are ready to go 🚀.\n\n## Usage\n\nThere are different ways to use Django prose according to your needs. We will examine all of them here.\n\n### Rendering rich-text in templates\n\nRich text content essentially is HTML. For this reason it needs to be manually marked as [`safe`](https://docs.djangoproject.com/en/4.2/ref/templates/builtins/#safe), when rendered in Django templates. Example:\n\n```django\n{{ document.content | safe}}\n```\n\n### Small rich-text content\n\nYou might want to add rich-text content in a model that is just a few characters (e.g. 140), like an excerpt from an article. In that case we suggest using the `RichTextField`. Example:\n\n```py\nfrom django.db import models\nfrom prose.fields import RichTextField\n\nclass Article(models.Model):\n    excerpt = RichTextField()\n```\n\nAs mentioned above, you need to mark the article excerpt as `safe`, in order to render it:\n\n```django\n<div class="article-excerpt">{{ article.excerpt | safe}}</div>\n```\n\n### Large rich-text content\n\nIn case you want to store large rich-text content, like the body of an article, which can span to quite a few thousand characters, we suggest you use the `AbstractDocument` model. This will save large rich-text content in a separate database table, which is better for performance. Example:\n\n```py\nfrom django.db import models\nfrom prose.fields import RichTextField\nfrom prose.models import AbstractDocument\n\nclass ArticleContent(AbstractDocument):\n    pass\n\nclass Article(models.Model):\n    excerpt = RichTextField()\n    body = models.OneToOneField(ArticleContent, on_delete=models.CASCADE)\n```\n\nSimilarly here as well, you need to mark the article\'s body as `safe`, in order to render it:\n\n```django\n<div class="article-body">{{ article.body.content | safe}}</div>\n```\n\n### Forms with rich-text editing\n\nYou can create forms for your Django Prose models, to provide rich-text editing functionality. In that case, you will also need to render `form.media`, to load Trix with its stylesheets.\n\n```django\n<form  method="POST" >\n  {% csrf_token %}\n  \n  {{ form.as_p }}\n  {{ form.media }}\n  \n  <button type="submit">Submit</button>\n</form>\n```\n\nThe same is true also, if you are rendering the forms field manually.\n\n### Attachments\n\nDjango Prose can also handle uploading attachments with drag and drop. To set this up, first you need to:\n\n- [x] Set up the `MEDIA_ROOT` and `MEDIA_URL` of your Django project (example in [`example/example/settings.py`](https://github.com/withlogicco/django-prose/blob/9e24cc794eae6db48818dd15a483d106d6a99da0/example/example/settings.py#L130-L131)))\n- [x] Include the Django Prose URLs (example in [`example/example/urls.py`](https://github.com/withlogicco/django-prose/blob/9e24cc794eae6db48818dd15a483d106d6a99da0/example/example/urls.py#L13-L14))\n- [x] (Optional) Set up a different Django storage to store your files (e.g. S3)\n\n### Full example\n\nYou can find a full example of a blog, built with Django Prose in the [`example`](./example/) directory.\n\n## 🔒 A note on security\n\nAs you can see in the examples above, what Django Prose does is provide you with a user friendly editor ([Trix](https://trix-editor.org/)) for your rich text content and then store it as HTML in your database. Since you will mark this HTML as safe in order to use it in your templates, it needs to be **sanitised**, before it gets stored in the database.\n\nFor this reason Django Prose is using [Bleach](https://bleach.readthedocs.io/en/latest/) to only allow the following tags and attributes:\n\n- **Allowed tags**: `p`, `ul`, `ol`, `li`, `strong`, `em`, `div`, `span`, `a`, `blockquote`, `pre`, `figure`, `figcaption`, `br`, `code`, `h1`, `h2`, `h3`, `h4`, `h5`, `h6`, `picture`, `source`, `img`\n- **Allowed attributes**: `alt`, `class`, `id`, `src`, `srcset`, `href`, `media`\n\n## Screenshots\n\n### Django Prose Documents in Django Admin\n\n![Django Prose Document in Django Admin](./docs/django-admin-prose-document.png)\n\n## Video tutorial\n\nIf you are more of a video person, you can watch our video showcasing how to **Create a blog using Django Prose** on YouTube\n\n[![Watch the video](https://img.youtube.com/vi/uICZjUpAbhQ/hqdefault.jpg)](https://youtu.be/uICZjUpAbhQ)\n\n## Real world use cases\n- **Remote Work Café**: Used to edit location pagess, like [Amsterdam | Remote Work Café](https://remotework.cafe/locations/amsterdam/)\n- In production by multiple clients of [LOGIC](https://withlogic.co), from small companies to the public sector.\n\nIf you are using Django Prose in your application too, feek free to open a [Pull Request](https://github.com/withlogicco/django-prose/pulls) to include it here. We would love to have it.\n\n## Development for Django Prose\n\nIf you plan to contribute code to Django Prose, this section is for you. All development tooling for Django Prose has been set up with Docker and Development Containers.\n\nTo get started run these commands in the provided order:\n\n```console\ndocker compose run --rm migrate\ndocker compose run --rm createsuperuser\ndocker compose up\n```\n\nIf you are using Visual Studio code, just open this repository in a container using the [`Dev Containers: Open Folder in Container`](https://code.visualstudio.com/docs/devcontainers/containers#_quick-start-open-an-existing-folder-in-a-container).\n\n---\n\n<p align="center">\n  <i>🦄 Built with <a href="https://withlogic.co/">LOGIC</a>. 🦄</i>\n</p>\n',
    'author': 'Paris Kasidiaris',
    'author_email': 'paris@withlogic.co',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/parisk/django-prose',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.forms.widgets import Textarea


class RichTextEditor(Textarea):
    template_name = "prose/forms/widgets/editor.html"

    class Media:
        css = {
            "all": (
                "https://cdnjs.cloudflare.com/ajax/libs/trix/1.3.0/trix.css",
                "prose/editor.css",
            ),
        }
        js = (
            "https://cdnjs.cloudflare.com/ajax/libs/trix/1.3.0/trix.js",
            "prose/editor.js",
        )
